//----------------------------------------------------------------------------------
// File:   BilateralBlurEngine.h
// Author: Miguel Sainz & Louis Bavoil
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma once

#include <DXUT.h>
#include <DXUTgui.h>
#include <DXUTmisc.h>
#include <SDKmisc.h>
#include "SimpleRT.h"
#include "SimpleDS.h"

class BilateralBlurEngine
{
public:
                    BilateralBlurEngine();

    HRESULT			OnCreateDevice      ( ID3D10Device* pd3dDevice );
    HRESULT			OnResizedSwapChain  ( int inputWidth, int inputHeight, int outputWidth, int outputHeight );
    void			OnReleasingSwapChain( );
    void			OnDestroyDevice     ( );
    
    HRESULT	        OnFrameRender( ID3D10RenderTargetView   *Destination, 
                                   ID3D10ShaderResourceView *Diffuse,
                                   ID3D10ShaderResourceView *Source, 
                                   ID3D10ShaderResourceView *SourceGuidance, 
                                   ID3D10ShaderResourceView *FullResDepth, 
                                   ID3D10ShaderResourceView *MultiSampleDepth);

    HRESULT	        OnFrameRender( ID3D10RenderTargetView   *Destination, 
                                   ID3D10ShaderResourceView *Diffuse );


    void            SetNumSamplesPerPixel(int numSamples);
    void            SetSceneScale(float scale);
    void			RecompileShader();
    void			CreateMenu( CDXUTDialog *myMenu );
    void			OnGUIEvent( CDXUTDialog *myMenu, UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );
 
private:
    void            UpdateBlur();
    void            UpdateSharpness();
    void            CreateDepthStencilStates();

protected:
    // Buffer sizes
    int				m_InWidth;
    int				m_InHeight;
    int				m_OutWidth;
    int				m_OutHeight;
    int             m_NumSamples;

    SimpleRT*		m_TempBuffer;
    SimpleRT*		m_EdgeBlurredAO;
    SimpleDS*		m_EdgeStencilAO;

    // D3D Device  
    ID3D10Device*	m_D3DDevice;
    D3D10_VIEWPORT  m_viewportInput;
    D3D10_VIEWPORT  m_viewportOutput;

    ID3D10DepthStencilState* m_DSStencilWrite;
    ID3D10DepthStencilState* m_DSStencilTest;
    ID3D10DepthStencilState* m_DSDefault;

    // The effects and rendering techniques
    ID3D10Effect*           m_Effect;
    ID3D10EffectTechnique*  m_TechniqueBlur;
    ID3D10EffectTechnique*  m_TechniqueBlurWithDiffuse;
    ID3D10EffectTechnique*  m_TechniqueBlur_SS;
    ID3D10EffectTechnique*  m_TechniqueBlurWithDiffuse_SS;
    ID3D10EffectTechnique*  m_TechniquePassthrough;
    ID3D10EffectTechnique*  m_TechniqueEdgeDetection;
    
    // Effect variable pointers
    ID3D10EffectShaderResourceVariable* m_SrcTexVar;   
    ID3D10EffectShaderResourceVariable* m_ColTexVar;
    ID3D10EffectShaderResourceVariable* m_DepTexVar;
    ID3D10EffectShaderResourceVariable* m_DepthBufferVar;
    ID3D10EffectShaderResourceVariable* m_MSAADepTexVar;
    
    ID3D10EffectScalarVariable*			m_pSharpness;
    ID3D10EffectScalarVariable*			m_pBlurRadius;
    ID3D10EffectScalarVariable*			m_pBlurFalloff;
    ID3D10EffectVectorVariable*			m_pResolution;
    ID3D10EffectVectorVariable*			m_pInvResolution;
    ID3D10EffectScalarVariable*			m_pEdgeThreshold;

    float                               m_SceneScale;
    float								m_Sharpness;
    float								m_BlurRadius;
    float								m_EdgeThreshold;
    float                               m_Resolution[2];   
    float                               m_InvResolution[2];
};
