//----------------------------------------------------------------------------------
// File:   RayMarchingAOEngine.h
// Author: Miguel Sainz &  Louis Bavoil
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma once

#include <DXUT.h>
#include <SDKmisc.h>
#include "ScreenSpaceAOEngine.h"

class RayMarchingAOEngine : public ScreenSpaceAOEngine
{
public:
                    RayMarchingAOEngine();
                    ~RayMarchingAOEngine();

    HRESULT			OnCreateDevice( ID3D10Device* pd3dDevice );
    HRESULT			OnDestroyDevice();
    HRESULT			OnResizedSwapChain( int width, int height, float fovy );
    void			OnReleasingSwapChain();
    
    HRESULT	        OnFrameRender( ID3D10ShaderResourceView *Depth, ID3D10ShaderResourceView *Normal = NULL  );

    void			RecompileShader();
    void			CreateMenu( CDXUTDialog *myMenu );
    void			OnGUIEvent( CDXUTDialog *myMenu, UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );

    void			UpdateDirs();

protected:
    // Buffer sizes
    int				m_BBWidth;
    int				m_BBHeight;

    // D3D Device  
    ID3D10Device*	m_D3DDevice;
    D3D10_VIEWPORT	m_Viewport;

    // The effects and rendering techniques
    ID3D10Effect          *m_Effect;
    ID3D10EffectTechnique *m_Technique_RAYMARCHING_AO;
    
    // Effect variable pointers
    ID3D10EffectShaderResourceVariable* m_DepTex;
    ID3D10EffectShaderResourceVariable* m_NorTex;
    ID3D10EffectShaderResourceVariable* m_RanTex;

    ID3D10EffectScalarVariable*			m_pNumSteps;
    ID3D10EffectScalarVariable*			m_pTraceRadius;
    ID3D10EffectScalarVariable*			m_pNumDirs;
    ID3D10EffectScalarVariable*			m_pNumRays;
    ID3D10EffectScalarVariable*			m_pLinAtt;
    ID3D10EffectScalarVariable*			m_pContrast;
    ID3D10EffectVectorVariable*			m_pFocalLen;
    ID3D10EffectVectorVariable*			m_pInvResolution;
    ID3D10EffectVectorVariable*			m_pDirsArray;

    int									m_NumSteps;
    float								m_TraceRadius;
    int									m_NumDirs;
    int									m_NumRays;
    float								m_FocalLen[2];
    float								m_InvResolution[2];
    float								m_Contrast;
    float								m_LinAtt;
    float								m_ZThresh;

    ID3D10Texture2D*					m_pRndTexture;
    ID3D10ShaderResourceView*			m_pRVRndTex;
};
