//----------------------------------------------------------------------------------
// File:    ResolveEngine.h
// Authors: Louis Bavoil & Miguel Sainz
// Email:   sdkfeedback@nvidia.com
// 
// Copyright (c) 2008 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma once

#include <DXUT.h>
#include <DXUTgui.h>
#include <DXUTmisc.h>
#include <SDKmisc.h>
#include "SimpleRT.h"
#include "ScreenSpaceAOPaths.h"

class ResolveEngine
{
public:
                    ResolveEngine();
                    ~ResolveEngine();

    HRESULT			OnCreateDevice      ( ID3D10Device* pd3dDevice );
    HRESULT			OnDestroyDevice     ( );
    HRESULT			OnResizedSwapChain  ( int inputWidth, int inputHeight, int nsamples, int outputWidth, int outputHeight, float zFar, float zNear );
    void			OnReleasingSwapChain( );

    HRESULT	        ResolveColors( SimpleRT*                  MRTColor,
                                   ID3D10ShaderResourceView** ResolvedColorSRV );

    HRESULT	        ResolveNormalsAndDepths( ID3D10ShaderResourceView*  MRTNormal,
                                             ID3D10ShaderResourceView*  MRTDepth,
                                             ID3D10ShaderResourceView*  DepthBuffer,
                                             ID3D10ShaderResourceView** ResolvedNormalSRV,
                                             ID3D10ShaderResourceView** ResolvedDepthSRV,
                                             ID3D10ShaderResourceView** LinearDepthSRV);

    void			RecompileShader();
    void			CreateMenu( CDXUTDialog *myMenu );
    void			OnGUIEvent( CDXUTDialog *myMenu, UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );

protected:
    // Buffer sizes
    int				m_inWidth;
    int				m_inHeight;
    int				m_outWidth;
    int				m_outHeight;
    int             m_BBNSamples;

    float			m_ZNear;
    float			m_ZFar;

    bool            m_NeedUpsampling;

    // Viewports
    D3D10_VIEWPORT  m_viewportInput;
    D3D10_VIEWPORT  m_viewportOutput;

    // Resolved textures
    SimpleRT*       m_ColorTexture;
    SimpleRT*       m_NormalTexture;
    SimpleRT*       m_SrcDepthTexture;
    SimpleRT*       m_DepthTexture;

    // D3D Device  
    ID3D10Device*	m_D3DDevice;
    D3D10_VIEWPORT	m_Viewport;

    // The effects and rendering techniques
    ID3D10Effect          *m_Effect;
    ID3D10EffectTechnique *m_TechniqueND;
    ID3D10EffectTechnique *m_TechniqueD;
    ID3D10EffectTechnique *m_TechniqueLinearizeD;
    
    ID3D10EffectShaderResourceVariable* m_NormalTex;
    ID3D10EffectShaderResourceVariable* m_DepthTex;
    ID3D10EffectScalarVariable*	m_pZNear;
    ID3D10EffectScalarVariable*	m_pZFar;

    ID3D10ShaderResourceView* LinearizeDepth(ID3D10ShaderResourceView* DepthBufferSRV);
    void ResolveDepthsAndNormals(ID3D10ShaderResourceView* FullResLinearDepth, ID3D10ShaderResourceView* MRTNormalSRV);
};
