//----------------------------------------------------------------------------------
// File:   Scene3D.h
// Author: Miguel Sainz
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------
#pragma once

#include <DXUT.h>
#include <SDKmisc.h>
#include <SDKmesh.h>
#include "ScreenSpaceAOPaths.h"

#define NUM_MESHES 7

struct MeshDescriptor
{
    WCHAR name[32];
    WCHAR path[96];
    bool  useGroundPlane;
    float scale;
};

class Scene3D
{
public:
    Scene3D();
    ~Scene3D();

    HRESULT OnCreateDevice      ( ID3D10Device* pd3dDevice, MeshDescriptor *meshDesc );
    HRESULT OnFrameRender       ( const D3DXMATRIX *p_mWorld, 
                                  const D3DXMATRIX *p_mView, 
                                  const D3DXMATRIX *p_mProj,  
                                  int modelID );
    HRESULT OnDestroyDevice     ( );

private:
    void DrawModel( int modelID );
    void DrawGroundPlane();
    void UpdateMatrices( const D3DXMATRIX &mWorld, const D3DXMATRIX &mView, const D3DXMATRIX &mProj );

    CDXUTSDKMesh  m_Mesh[NUM_MESHES];
    bool          m_useGroundPlane[NUM_MESHES];

    D3DXVECTOR3 m_bbExtents[NUM_MESHES];
    D3DXVECTOR3 m_bbCenter[NUM_MESHES]; 

    // Geometry
    ID3D10InputLayout          *m_VertexLayout;
    ID3D10Buffer               *m_VertexBuffer;
    ID3D10Buffer               *m_IndexBuffer;
    
    // The device
    ID3D10Device               *m_D3DDevice;
    
    // The effects and rendering techniques
    ID3D10Effect               *m_Effect;
    ID3D10EffectTechnique      *m_TechniqueSceneC;
    ID3D10EffectTechnique      *m_TechniqueSceneCN;
    ID3D10EffectTechnique      *m_TechniqueSceneCD;
    ID3D10EffectTechnique      *m_TechniqueSceneCND;
    ID3D10EffectTechnique      *m_Technique;
    
    // Effect variable pointers
    ID3D10EffectMatrixVariable *m_VarWVP;
    ID3D10EffectMatrixVariable *m_VarWV;
    ID3D10EffectMatrixVariable *m_VarWVIT;
    ID3D10EffectVectorVariable *m_VarEye;
    ID3D10EffectScalarVariable *m_VarIsGroundPlane;

};
