//----------------------------------------------------------------------------------
// File:   ScreenSpaceAOEngine.h
// Author: Miguel Sainz
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma once

#include <DXUT.h>
#include <DXUTgui.h>
#include <DXUTmisc.h>
#include "SimpleRT.h"
#include "ScreenSpaceAOPaths.h"

class ScreenSpaceAOEngine
{
public:
    virtual HRESULT OnCreateDevice      ( ID3D10Device* pd3dDevice ) = 0;
    virtual HRESULT OnDestroyDevice     ( ) = 0;
    virtual HRESULT OnResizedSwapChain  ( int width, int height, float fovy) = 0;
    virtual void    OnReleasingSwapChain( ) = 0;

    virtual HRESULT OnFrameRender       ( ID3D10ShaderResourceView *Depth, ID3D10ShaderResourceView *Normal = NULL ) = 0;

    virtual void RecompileShader        () {};

    // Menu related stuff
    virtual void CreateMenu ( CDXUTDialog *myMenu ) = 0;
    virtual void OnGUIEvent ( CDXUTDialog *myMenu, UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext ) = 0;

    void  SetSceneScale(float val){  m_AORadius = val; }
    float GetSceneScale()         { return m_AORadius; }

    SimpleRT *getAmbientOcclusionRT()    { return m_SSAOBuffer; }

protected:
    SimpleRT *m_SSAOBuffer;

    float    m_AORadius;
};
