//----------------------------------------------------------------------------------
// File:   SimpleDS.h
// Author: Louis Bavoil
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2008 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma once

class SimpleDS
{
public:
    ID3D10Texture2D* pTexture;
    ID3D10DepthStencilView* pDSV;
    ID3D10ShaderResourceView* pSRV;

    SimpleDS( ID3D10Device* pd3dDevice, D3D10_TEXTURE2D_DESC* pTexDesc )
    {
        pTexture = NULL;
        pDSV     = NULL;
        pSRV     = NULL;

        HRESULT hr;
        V( pd3dDevice->CreateTexture2D( pTexDesc, NULL, &pTexture ) );
            
        D3D10_DEPTH_STENCIL_VIEW_DESC dsvDesc;
        D3D10_SHADER_RESOURCE_VIEW_DESC srvDesc;

        switch(pTexDesc->Format)
        {
            case DXGI_FORMAT_R32_TYPELESS:
                dsvDesc.Format = DXGI_FORMAT_D32_FLOAT;
                srvDesc.Format = DXGI_FORMAT_R32_FLOAT;
                break;
            case DXGI_FORMAT_D24_UNORM_S8_UINT:
                dsvDesc.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
                srvDesc.Format = DXGI_FORMAT_R24_UNORM_X8_TYPELESS;
                break;
        }  
        dsvDesc.ViewDimension = D3D10_DSV_DIMENSION_TEXTURE2D;
        dsvDesc.Texture2D.MipSlice = 0;
        V( pd3dDevice->CreateDepthStencilView( pTexture, &dsvDesc, &pDSV ) );

        if( pTexDesc->Format == DXGI_FORMAT_R32_TYPELESS)
        {
            srvDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
            srvDesc.Texture2D.MostDetailedMip = 0;
            srvDesc.Texture2D.MipLevels = 1;
            V( pd3dDevice->CreateShaderResourceView( pTexture, &srvDesc, &pSRV ) );
        }
    }

    ~SimpleDS()
    {
        SAFE_RELEASE( pTexture );
        SAFE_RELEASE( pDSV );
        SAFE_RELEASE( pSRV );
    }
};
